@@ -60,8 +60,11 @@ struct GpmBrightnessXRandRPrivate
 	Display			*dpy;
 	guint			 shared_value;
 	gboolean		 has_extension;
+#if (RANDR_MAJOR == 1 && RANDR_MINOR >= 3)
+	gboolean		 has_randr13;
+#endif
 	gboolean		 hw_changed;
-	/* An cache of XRRScreenResources is used as XRRGetScreenResources is expensive */
+	/* A cache of XRRScreenResources is used as XRRGetScreenResources is expensive */
 	GPtrArray		*resources;
 };
 
@@ -119,7 +122,6 @@ gpm_brightness_xrandr_output_set_internal (GpmBrightnessXRandR *brightness, RROu
 	gboolean ret = TRUE;
 
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_XRANDR (brightness), FALSE);
-	g_return_val_if_fail (value >= 0, FALSE);
 
 	/* don't abort on error */
 	gdk_error_trap_push ();
@@ -171,6 +173,34 @@ gpm_brightness_xrandr_setup_display (GpmBrightnessXRandR *brightness)
 	return TRUE;
 }
 
+#if (RANDR_MAJOR == 1 && RANDR_MINOR >= 3)
+/**
+ * gpm_brightness_xrandr_setup_version: Check whether xserver really supports xrandr-1.3 features.
+ **/
+static gboolean
+gpm_brightness_xrandr_setup_version (GpmBrightnessXRandR *brightness)
+{
+	gint major, minor;
+
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS_XRANDR (brightness), FALSE);
+
+	/* get the display */
+	brightness->priv->dpy = GDK_DISPLAY();
+	if (!brightness->priv->dpy) {
+		egg_error ("Cannot open display");
+		return FALSE;
+	}
+	if (!XRRQueryVersion (brightness->priv->dpy, &major, &minor)) {
+		return FALSE;
+	}
+	if (major == 1 && minor < 3) {
+		egg_debug ("RandR version %d.%d does not support XRRGetScreenResourcesCurrent", major, minor);
+		return FALSE;
+	}
+	return TRUE;
+}
+#endif
+
 /**
  * gpm_brightness_xrandr_output_get_limits:
  **/
@@ -233,6 +263,7 @@ static gboolean
 gpm_brightness_xrandr_output_down (GpmBrightnessXRandR *brightness, RROutput output)
 {
 	guint cur;
+	guint step;
 	gboolean ret;
 	guint min, max;
 
@@ -249,10 +280,12 @@ gpm_brightness_xrandr_output_down (GpmBrightnessXRandR *brightness, RROutput out
 		egg_debug ("already min");
 		return TRUE;
 	}
-	cur -= gpm_brightness_get_step ((max-min)+1);
-	if (cur < min) {
+	step = gpm_brightness_get_step ((max-min)+1);
+	if (cur < step) {
 		egg_debug ("truncating to %i", min);
 		cur = min;
+	} else {
+		cur -= step;
 	}
 	ret = gpm_brightness_xrandr_output_set_internal (brightness, output, cur);
 	return ret;
@@ -316,17 +349,17 @@ gpm_brightness_xrandr_output_set (GpmBrightnessXRandR *brightness, RROutput outp
 	egg_debug ("percent=%i, absolute=%i", brightness->priv->shared_value, shared_value_abs);
 
 	egg_debug ("hard value=%i, min=%i, max=%i", cur, min, max);
-	if (shared_value_abs > max)
+	if (shared_value_abs > (gint) max)
 		shared_value_abs = max;
-	if (shared_value_abs < min)
+	if (shared_value_abs < (gint) min)
 		shared_value_abs = min;
-	if (cur == shared_value_abs) {
+	if ((gint) cur == shared_value_abs) {
 		egg_debug ("already set %i", cur);
 		return TRUE;
 	}
 
 	/* step the correct way */
-	if (cur < shared_value_abs) {
+	if ((gint) cur < shared_value_abs) {
 
 		/* some adaptors have a large number of steps */
 		step = gpm_brightness_get_step (shared_value_abs - cur);
@@ -337,7 +370,7 @@ gpm_brightness_xrandr_output_set (GpmBrightnessXRandR *brightness, RROutput outp
 			ret = gpm_brightness_xrandr_output_set_internal (brightness, output, i);
 			if (!ret)
 				break;
-			if (cur != shared_value_abs)
+			if ((gint) cur != shared_value_abs)
 				g_usleep (1000 * GPM_BRIGHTNESS_DIM_INTERVAL);
 		}
 	} else {
@@ -351,7 +384,7 @@ gpm_brightness_xrandr_output_set (GpmBrightnessXRandR *brightness, RROutput outp
 			ret = gpm_brightness_xrandr_output_set_internal (brightness, output, i);
 			if (!ret)
 				break;
-			if (cur != shared_value_abs)
+			if ((gint) cur != shared_value_abs)
 				g_usleep (1000 * GPM_BRIGHTNESS_DIM_INTERVAL);
 		}
 	}
@@ -364,16 +397,17 @@ gpm_brightness_xrandr_output_set (GpmBrightnessXRandR *brightness, RROutput outp
 static gboolean
 gpm_brightness_xrandr_foreach_resource (GpmBrightnessXRandR *brightness, GpmXRandROp op, XRRScreenResources *resources)
 {
-	guint i;
+	gint i;
 	gboolean ret;
 	gboolean success_any = FALSE;
+	RROutput output;
 
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_XRANDR (brightness), FALSE);
 
 	/* do for each output */
 	for (i=0; i<resources->noutput; i++) {
+		output = resources->outputs[i];
 		egg_debug ("resource %i of %i", i+1, resources->noutput);
-		RROutput output = resources->outputs[i];
 		if (op==ACTION_BACKLIGHT_GET) {
 			ret = gpm_brightness_xrandr_output_get_percentage (brightness, output);
 		} else if (op==ACTION_BACKLIGHT_INC) {
@@ -598,21 +632,32 @@ gpm_brightness_xrandr_update_cache (GpmBrightnessXRandR *brightness)
 	/* do for each screen */
 	display = gdk_display_get_default ();
 	length = ScreenCount (brightness->priv->dpy);
-	for (screen = 0; screen < length; screen++) {
+	for (screen = 0; screen < (gint) length; screen++) {
 		egg_debug ("screen %i of %i", screen+1, length);
 		gscreen = gdk_display_get_screen (display, screen);
 
 		/* if we have not setup the changed on the monitor, set it here */
 		if (g_object_get_data (G_OBJECT (gscreen), "gpk-set-monitors-changed") == NULL) {
 			egg_debug ("watching ::monitors_changed on %p", gscreen);
-			g_object_set_data (G_OBJECT (gscreen), "gpk-set-monitors-changed", "true");
+			g_object_set_data (G_OBJECT (gscreen), "gpk-set-monitors-changed", (gpointer) "true");
 			g_signal_connect (G_OBJECT (gscreen), "monitors_changed",
 					  G_CALLBACK (gpm_brightness_monitors_changed), brightness);
 		}
 
-		/* this is slow */
 		root = RootWindow (brightness->priv->dpy, screen);
+		/* XRRGetScreenResourcesCurrent is less expensive than
+		   XRRGetScreenResources, however it is available only
+		   in RandR 1.3 or higher and of course xserver needs
+		   to support it.
+		*/
+#if (RANDR_MAJOR == 1 && RANDR_MINOR >= 3)
+		if (brightness->priv->has_randr13)
+			resource = XRRGetScreenResourcesCurrent (brightness->priv->dpy, root);
+		else
+			resource = XRRGetScreenResources (brightness->priv->dpy, root);
+#else
 		resource = XRRGetScreenResources (brightness->priv->dpy, root);
+#endif
 		if (resource != NULL) {
 			egg_debug ("adding resource %p", resource);
 			g_ptr_array_add (brightness->priv->resources, resource);
@@ -675,8 +720,13 @@ gpm_brightness_xrandr_init (GpmBrightnessXRandR *brightness)
 
 	/* can we do this */
 	brightness->priv->has_extension = gpm_brightness_xrandr_setup_display (brightness);
-	if (brightness->priv->has_extension == FALSE)
+	if (brightness->priv->has_extension == FALSE) {
+		egg_debug ("no XRANDR extension, so aborting init");
 		return;
+	}
+#if (RANDR_MAJOR == 1 && RANDR_MINOR >= 3)
+	brightness->priv->has_randr13 = gpm_brightness_xrandr_setup_version (brightness);
+#endif
 
 	screen = gdk_screen_get_default ();
 	window = gdk_screen_get_root_window (screen);
